"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const broccoli_funnel_1 = __importDefault(require("broccoli-funnel"));
const broccoli_debug_1 = __importDefault(require("broccoli-debug"));
const broccoli_babel_transpiler_1 = __importDefault(require("broccoli-babel-transpiler"));
const broccoli_merge_trees_1 = __importDefault(require("broccoli-merge-trees"));
const path_1 = __importDefault(require("path"));
const namespace_imports_1 = require("./namespace-imports");
const namespace_templates_1 = require("./namespace-templates");
const namespace_addon_imports_babel_1 = __importDefault(require("../plugins/namespace-addon-imports-babel"));
const walk_sync_1 = __importDefault(require("walk-sync"));
const resolvablePatterns = ['helpers/**', 'components/**', 'modifiers/**', 'templates/components/**'];
/**
 * Discover template resolvable names by walking through folders that
 * contain resolvable modules.
 */
function templateResolvableNames(rootPath) {
    const resolvableNamesAndDirs = walk_sync_1.default(path_1.default.join(rootPath, 'app'), { globs: resolvablePatterns });
    const resolvableNames = [];
    for (let resolvableNameOrDir of resolvableNamesAndDirs) {
        let parts = path_1.default.parse(resolvableNameOrDir);
        if (parts.dir && parts.name) {
            resolvableNames.push(parts.name);
        }
    }
    return resolvableNames;
}
exports.templateResolvableNames = templateResolvableNames;
function namespaceAddon(addonVersionSummary) {
    // console.log('namespaceAddon:', addonVersionSummary.version);
    for (let addonInstance of addonVersionSummary.instances) {
        const { name } = addonInstance;
        const { cacheKey } = addonVersionSummary;
        // console.log('namespacing instance:', name, cacheKey);
        const originalTreeGenerator = addonInstance.treeGenerator;
        addonInstance.treeGenerator = function (fullPath) {
            let pathSegments = fullPath.split(path_1.default.sep);
            let type = pathSegments[pathSegments.length - 1];
            if (type === 'templates') {
                type = pathSegments[pathSegments.length - 2] + '/' + type;
            }
            // console.log('treeGenerator', name, cacheKey, type);
            let result = originalTreeGenerator.call(this, fullPath);
            if (type === 'addon' || type === 'addon/templates') {
                /**
                 * Namespace any of this addon's modules in the `addon` tree according
                 * to its `cacheKey`.
                 *
                 * A given helper, such as:
                 *   `my-addon/helpers/addon-version`
                 *
                 * Will be namespaced in the `addon` tree as:
                 *   `my-addon/ns/bfc08628a53622a84307c3864928b5a2/helpers/addon-version`
                 */
                let debugTree = new broccoli_debug_1.default(result, `treeGenerator:${name}:${cacheKey}:${type}:input`);
                let output = new broccoli_funnel_1.default(debugTree, {
                    destDir: `ns/${cacheKey}`
                });
                result = new broccoli_debug_1.default(output, `treeGenerator:${name}:${cacheKey}:${type}:output`);
            }
            else if (type === 'app') {
                /**
                 * Namespace any of this addon's modules in the `app` tree according to
                 * the pattern that will be used for resolution.
                 *
                 * A given helper, such as:
                 *   `my-addon/helpers/addon-version`
                 *
                 * Will be namespaced in the app tree as:
                 *   `host-app/helpers/addon-version-bfc08628a53622a84307c3864928b5a2`
                 *
                 * So that it will be properly resolved when it appears in templates as:
                 *   `{{addon-version-bfc08628a53622a84307c3864928b5a2}}`
                 */
                let unresolvedThings = new broccoli_funnel_1.default(result, {
                    exclude: resolvablePatterns,
                });
                let debugTree = new broccoli_debug_1.default(result, `treeGenerator:${name}:${cacheKey}:${type}:input`);
                let resolvedThings = new broccoli_funnel_1.default(debugTree, {
                    include: resolvablePatterns,
                    getDestinationPath(relativePath) {
                        let parts = path_1.default.parse(relativePath);
                        return `${parts.dir}/${parts.name}-${cacheKey}${parts.ext}`;
                    }
                });
                let resolvedThingsOutput = new broccoli_debug_1.default(resolvedThings, `treeGenerator:${name}:${cacheKey}:${type}:output`);
                let resolvedThingsNamespaced = broccoli_babel_transpiler_1.default(resolvedThingsOutput, {
                    plugins: [[
                            namespace_addon_imports_babel_1.default(),
                            { name, cacheKey, appOrAddonName: addonInstance.name },
                            `${name}-${cacheKey}-${addonInstance.name}`
                        ]]
                });
                let resolvedThingsNamespacedOutput = new broccoli_debug_1.default(resolvedThingsNamespaced, `treeGenerator:${name}:${cacheKey}:${type}:namespaced`);
                result = new broccoli_merge_trees_1.default([unresolvedThings, resolvedThingsNamespacedOutput]);
            }
            return result;
        };
        const addonParent = addonInstance.parent === addonInstance.project ? addonInstance.app : addonInstance.parent;
        // Namespace any imports in the addon itself and its parent
        namespace_imports_1.namespaceImports(addonInstance, name, cacheKey);
        namespace_imports_1.namespaceImports(addonParent, name, cacheKey);
        // Discover template resolvable names by walking through folders that
        // contain resolvable modules.
        const templateNamespacingOptions = {
            namespace: cacheKey,
            names: templateResolvableNames(addonInstance.root)
        };
        // Namespace any resolvable names in the addon's templates or in its
        // parent's templates
        namespace_templates_1.namespaceTemplates(addonInstance, templateNamespacingOptions);
        namespace_templates_1.namespaceTemplates(addonParent, templateNamespacingOptions);
    }
}
exports.default = namespaceAddon;
//# sourceMappingURL=namespace-addon.js.map