"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const is_runtime_addon_1 = __importDefault(require("./is-runtime-addon"));
const validate_cache_key_dependency_1 = require("./validate-cache-key-dependency");
/**
 * Given a Project instance, traverses the addon inclusion tree to discover all
 * included versions of all addons in the build, producing a structure in the
 * format:
 *
 *  {
 *    "addon-name": {
 *      "addon.version": [
 *        ["project", "immediate-dependent"],
 *        ["project", "path-to", "nested-dependent"]
 *      ]
 *    }
 *  }
 */
function reviewProject(project, options = {}) {
    options.namespaceAddons = options.namespaceAddons || [];
    options.ignoreAddons = IGNORED_ADDONS.concat(options.ignoreAddons || []);
    const addons = {};
    const errors = [];
    const summary = { addons, errors };
    traverseAddons([project.name()], project.addons, summary, options);
    if (options.conflictsOnly) {
        for (const name in addons) {
            if (Object.keys(addons[name]).length < 2) {
                delete addons[name];
            }
        }
    }
    if (!options.skipCacheKeyDependencyChecks) {
        const version = validate_cache_key_dependency_1.cacheKeyDependencyVersion(project.dependencies(), project.root);
        if (version && !validate_cache_key_dependency_1.validateCacheKeyDependency(version)) {
            summary.errors.push(`This project has a dependency on 'calculate-cache-key-for-tree@${version}'. Update to v1.2.3 or later to avoid unnecessary addon duplication.`);
        }
    }
    return summary;
}
exports.default = reviewProject;
;
function traverseAddons(parentPath, addons, summary, options) {
    for (const addon of addons) {
        const name = addon.pkg.name;
        const runtime = is_runtime_addon_1.default(addon);
        if (!options.ignoreAddons.includes(name) &&
            (!options.runtimeOnly || runtime)) {
            const version = addon.pkg.version;
            if (version) {
                const cacheKey = (addon.cacheKeyForTree && addon.cacheKeyForTree()) || version;
                const addonSummary = summary.addons[name] || (summary.addons[name] = Object.create(null));
                const keyedSummary = addonSummary[cacheKey] || (addonSummary[cacheKey] = {
                    version,
                    cacheKey,
                    runtime,
                    dependents: []
                });
                keyedSummary.dependents.push(parentPath);
                if (options.namespaceAddons.includes(name)) {
                    keyedSummary.instances = keyedSummary.instances || [];
                    keyedSummary.instances.push(addon);
                }
            }
            if (!options.skipCacheKeyDependencyChecks && addon.dependencies) {
                const version = validate_cache_key_dependency_1.cacheKeyDependencyVersion(addon.dependencies(), addon.root);
                if (version && !validate_cache_key_dependency_1.validateCacheKeyDependency(version)) {
                    summary.errors.push(`The addon '${name}' has a dependency on 'calculate-cache-key-for-tree@${version}'. Update to v1.2.3 or later to avoid unnecessary addon duplication.`);
                }
            }
        }
        if (addon.addons) {
            traverseAddons(parentPath.concat(name), addon.addons, summary, options);
        }
    }
}
const IGNORED_ADDONS = [
    'ember-cli-htmlbars',
    'ember-cli-babel',
    'ember-cli-sass',
    'ember-cli-node-assets',
    'ember-cli-htmlbars-inline-precompile',
    'ember-auto-import',
    'ember-cli-typescript'
];
//# sourceMappingURL=review-project.js.map